'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var core = require('@babel/core');
var template = require('@babel/template');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

var template__default = /*#__PURE__*/_interopDefault(template);

const buildSetFileScope = template__default['default'](`
  import { setFileScope, endFileScope } from %%packageIdentifier%%
  setFileScope(%%fileScope%%)
`);
const debuggableFunctionConfig = {
  style: {
    maxParams: 2
  },
  createTheme: {
    maxParams: 3
  },
  mapToStyles: {
    maxParams: 3
  },
  fontFace: {
    maxParams: 2
  },
  keyframes: {
    maxParams: 2
  },
  createVar: {
    maxParams: 1
  }
};
const styleFunctions = [...Object.keys(debuggableFunctionConfig), 'globalStyle', 'createGlobalTheme', 'createThemeVars', 'globalFontFace', 'globalKeyframes'];

const extractName = node => {
  if (core.types.isObjectProperty(node) && core.types.isIdentifier(node.key)) {
    return node.key.name;
  } else if ((core.types.isVariableDeclarator(node) || core.types.isFunctionDeclaration(node)) && core.types.isIdentifier(node.id)) {
    return node.id.name;
  } else if (core.types.isExportDefaultDeclaration(node)) {
    return 'default';
  } else if (core.types.isVariableDeclarator(node) && core.types.isArrayPattern(node.id) && core.types.isIdentifier(node.id.elements[0])) {
    return node.id.elements[0].name;
  }
};

const getDebugId = path => {
  const {
    parent
  } = path;

  if (core.types.isObjectProperty(parent) || core.types.isReturnStatement(parent) || core.types.isArrowFunctionExpression(parent) || core.types.isArrayExpression(parent) || core.types.isSpreadElement(parent)) {
    const names = [];
    path.findParent(({
      node: parentNode
    }) => {
      const name = extractName(parentNode);

      if (name) {
        names.unshift(name);
      } // Traverse all the way to the root


      return false;
    });
    return names.join('_');
  } else {
    return extractName(parent);
  }
};

const getRelevantCall = (node, namespaceImport, importIdentifiers) => {
  const {
    callee
  } = node;

  if (namespaceImport && core.types.isMemberExpression(callee) && core.types.isIdentifier(callee.object, {
    name: namespaceImport
  })) {
    return styleFunctions.find(exportName => core.types.isIdentifier(callee.property, {
      name: exportName
    }));
  } else {
    const importInfo = Array.from(importIdentifiers.entries()).find(([identifier]) => core.types.isIdentifier(callee, {
      name: identifier
    }));

    if (importInfo) {
      return importInfo[1];
    }
  }
};

function index () {
  return {
    pre({
      opts
    }) {
      var _this$opts, _this$opts2;

      this.importIdentifiers = new Map();
      this.namespaceImport = '';
      this.packageIdentifier = ((_this$opts = this.opts) === null || _this$opts === void 0 ? void 0 : _this$opts.alias) || '@vanilla-extract/css';
      const projectRoot = ((_this$opts2 = this.opts) === null || _this$opts2 === void 0 ? void 0 : _this$opts2.projectRoot) || opts.root;

      if (!projectRoot) {
        // TODO Make error better
        throw new Error('Project root must be specified');
      }

      if (!opts.filename) {
        // TODO Make error better
        throw new Error('Filename must be available');
      }

      this.fileScope = path.relative(projectRoot, opts.filename);
    },

    visitor: {
      Program: {
        exit(path) {
          if (this.importIdentifiers.size > 0 || this.namespaceImport) {
            // Wrap module with file scope calls
            path.unshiftContainer('body', buildSetFileScope({
              packageIdentifier: core.types.stringLiteral(`${this.packageIdentifier}/fileScope`),
              fileScope: core.types.stringLiteral(this.fileScope)
            }));
            path.pushContainer('body', core.types.callExpression(core.types.identifier('endFileScope'), []));
          }
        }

      },

      ImportDeclaration(path) {
        if (path.node.source.value === this.packageIdentifier) {
          path.node.specifiers.forEach(specifier => {
            if (core.types.isImportNamespaceSpecifier(specifier)) {
              this.namespaceImport = specifier.local.name;
            } else if (core.types.isImportSpecifier(specifier)) {
              const {
                imported,
                local
              } = specifier;
              const importName = core.types.isIdentifier(imported) ? imported.name : imported.value;

              if (styleFunctions.includes(importName)) {
                this.importIdentifiers.set(local.name, importName);
              }
            }
          });
        }
      },

      CallExpression(path) {
        const {
          node
        } = path;
        const usedExport = getRelevantCall(node, this.namespaceImport, this.importIdentifiers);

        if (usedExport && usedExport in debuggableFunctionConfig) {
          if (node.arguments.length < debuggableFunctionConfig[usedExport].maxParams) {
            const debugIdent = getDebugId(path);

            if (debugIdent) {
              node.arguments.push(core.types.stringLiteral(debugIdent));
            }
          }
        }
      }

    }
  };
}

exports.default = index;
